%% Compressed Diffusion LMS

% Authors: Ibrahim El khalil Harrane, R?my Flamary, C?dric Richard
% Please cite the paper refrenced below if you use this code.
% This code corresponds to the first experimentation of the paper
% (Doubly-compressed diffion LMS curves).

% @article{harranedoubly,
%   title={Doubly compressed diffusion LMS over adaptive networks},
%   author={Harrane, Ibrahim El Khalil and Flamary, R{\'e}mi and Richard, C{\'e}dric}


%%
clear all
close all
addpath(genpath('.'));
addpath(genpath('../toolbox'));

%% Simulation settings
rng(356);


L=5;                                    % w_o length
N=10;                                   % number of nodes
M=3;                                    % number of shared enries of the local estimates 
N_iter=2e4;                             % number of iteration
N_exp=100;                              % number of experimentation
step_size=1e-3;                         % step size 
mu=step_size*ones(N,1);
sigma_u=1e0;                            % regression vector variation
Sigma_u=rand(N,1)+1;                    % regression vector variation 
sigma_v=1e-3;                           % noise variation
Sigma_v=sigma_v*ones(N,1);  
r=1;                                    % distance parameter for network topology
w_star=randn(L,1);                      % local estimates initialisation
w_0=repmat(randn(L,1),1,N);             % desired parameter selection



%%


options.M=L;
options.N=N;
options.N_iter=N_iter;
options.parameter=r;
options.corr=0;
options.type='radial';
options.wo=w_star;
options.rule='metropolis';
options.sigma_u=sigma_u;
options.Sigma_u=Sigma_u;
options.sigma_v=sigma_v;
options.one_iter=0;
options.rtype='provided';
load('seed_10.mat');

%%

[X,L1,Ac,xy] = get_network( options);
[ Coef ] = get_coef( X,Sigma_u,Sigma_v,mu,options );
clf
print_network( xy,X,options )
C=Coef;
A=eye(N);






%% Monte-Carlo runs 

e_compressed=zeros(N_iter,N);
e_t=zeros(N_iter,N);
W_t=zeros(N_iter,L);
W_tot=zeros(N_iter,L);
for l=1:N_exp
    
    fprintf('exp n %d  \n',l);
    [ d,u,Sigma_u,Sigma_v,w_star] = get_data( options );

    u=permute(u,[2 3 1]);
    
    
    
    % generating H for compressed Diffusion 
    
    H=zeros(L,N,N_iter);
    H_grad=zeros(L,N,N_iter);
    for i=1:N
        for j=1:N_iter
            H(:,i,j)=randperm(L);
        end
    end
    
    
        % Compressed Diff ATC
        [e_t,w_k_compressed]=compressed_diffusion(eye(N),C,w_0,u,d,mu,H,M,w_star);
        
    

    e_compressed=e_compressed+e_t;
    
end

MSD=mean(e_compressed,2)/N_exp;

%% theoretical model
fprintf('Theoritical model \n');

C_i=kron(C,eye(L));
R_u=[];
R=[];
S=[];
R_v=[];
M_mu=[];



Ru=zeros(L,L,N);


for k=1:N
    Ru(:,:,k)=Sigma_u(k)*eye(L);
    R_u=blkdiag(R_u,Ru(:,:,k));
end



for i=1:N
    R_ut=0;
    for j=1:N
        R_ut=R_ut+C(j,i)*Ru(:,:,j);
    end
    M_mu=blkdiag(M_mu,mu(i)*eye(L));
    R=blkdiag(R,R_ut);
    R_v=blkdiag(R_v,Sigma_v(i));
    S=blkdiag(S,Sigma_v(i)*Ru(:,:,i));
end


%% Mean square stability



H_p=eye(L*N);

P1_blk=0;
P1_diag=0;


P2_diag=0;


P3_diag=0;


P4_diag=0;

for i=1:N*L
    G=diag(H_p(:,i));
    
    P1_diag=P1_diag+kron(G*R*M_mu,G*R*M_mu);
    
    
end


for i=1:N*L
    G=diag(H_p(:,i));
    
    
    for j=i:L:L*N
        Gj=diag(H_p(:,j));
        P2_diag=P2_diag+kron(G*R_u*C_i*M_mu,Gj*R*M_mu)...`
            +kron(Gj*R_u*C_i*M_mu,G*R*M_mu);
        P3_diag=P3_diag+kron(G*R*M_mu,Gj*R_u*C_i*M_mu)...
            + kron(Gj*R*M_mu,G*R_u*C_i*M_mu);
        
        P4_diag=P4_diag+kron(G*R_u*C_i*M_mu,Gj*R_u*C_i*M_mu)...
            + kron(Gj*R_u*C_i*M_mu,G*R_u*C_i*M_mu);
    end
    P2_diag=P2_diag-kron(G*R_u*C_i*M_mu,G*R*M_mu);
    P3_diag=P3_diag-kron(G*R*M_mu,G*R_u*C_i*M_mu);
    P4_diag=P4_diag-kron(G*R_u*C_i*M_mu,G*R_u*C_i*M_mu);
end


M_blk=0;
for i=1:N
    Gt=zeros(N);
    Gt(i,i)=1;
    G=kron(Gt,eye(L));
    
    P1_blk=P1_blk+kron(G*R*M_mu,G*R*M_mu);
end


alpha_1=(M/L)*((M-1)/(L-1)-(M/L));
alpha_2=(M/L)*(1-(M-1)/(L-1));
alpha_3=(M/L)^2;

c1=(M/L)*(1-(M-1)/(L-1));
c2=(M/L)*(M-1)/(L-1);

P1= alpha_1*P1_blk + alpha_2*P1_diag + alpha_3*kron(R*M_mu,R*M_mu);
P2= (M/L - c2) * kron(R_u*C_i*M_mu, R*M_mu) - c1 * P2_diag ;
P3= (M/L - c2) * kron(R*M_mu, R_u*C_i*M_mu) - c1 * P3_diag ;
P4= (1 - 2* M/L +c2) * kron(R_u*C_i*M_mu,R_u*C_i*M_mu) + c1 * P4_diag ;

I_NM=eye(L*N);
x=M/L;


F=eye((N*L)^2)-x*kron(R*M_mu,eye(N*L))-(1-x)*kron(R_u*C_i*M_mu,eye(N*L))...
    -x*kron(I_NM,R*M_mu)...
    -(1-x)*kron(I_NM,R_u*C_i*M_mu)...
    +P1+P2+P3+P4;





F=sparse(F);


G_i=M_mu*C_i';
vec_y=reshape(G_i*S'*G_i',1,[]);



sig=eye(N*L);
epsilon=zeros(N_iter,1);

wt_1=repmat(w_star,1,N) - w_0;
epsilon_t=(1/N)*wt_1(:)'*reshape(F*sig(:),N*L,N*L)*wt_1(:)+(1/N)*vec_y*sig(:);
epsilon(1)=epsilon_t;

Fi=F;


for i=2:N_iter
    
    fprintf('iter n %d  \n',i);
    
    Met=(eye((N*L)^2)-F)*Fi*sig(:);
    Met=reshape(Met,N*L,N*L);
    
    epsilon_t=epsilon_t+(1/N)*vec_y*Fi*sig(:)-(1/N)*wt_1(:)'*Met*wt_1(:);
    epsilon(i)=epsilon_t;
    
    Fi=Fi*F;
end

MSD_t=(1/N)*vec_y*(eye(size(F))-F)^(-1)*reshape(eye(N*L),[],1);
EMSE=(1/N)*vec_y*(eye(size(F))-F)^(-1)*R(:);




%% MSD_curves

clf
fs=16;
plot(10*log10([MSD epsilon MSD_t*ones(N_iter,1)])); grid on
axis([0 N_iter -70 20])
title({'Compressed diffusion LMS'},...
    'Interpreter','latex','FontSize',1.2*fs)
legend( 'Theoretical MSD', 'Simulated MSD', 'Steady state MSD')
set(gca,'fontsize',16)
xlabel('iteration','fontsize',20,'interpreter','Latex');
ylabel('MSD (dB)','fontsize',20,'interpreter','Latex');



