%% Compressed Diffusion LMS

% Authors: Ibrahim El khalil Harrane, R?my Flamary, C?dric Richard
% Please cite the paper refrenced below if you use this code.
%   This code corresponds to the first experimentation of the paper (Doubly-compressed diffion LMS curves)

% @article{harranedoubly,
%   title={Doubly compressed diffusion LMS over adaptive networks},
%   author={Harrane, Ibrahim El Khalil and Flamary, R{\'e}mi and Richard, C{\'e}dric}

%%
clear all
close all
addpath(genpath('.'));
addpath(genpath('../toolbox'));

%% Simulation settings

rng(356);


L=5;                            % w_o length
N=10;                           % number of nodes
M=3;                            % number of shared enries of the local estimates 
M_grad=1;                       % number of shared enries of the gradients 
N_iter=3e4;                     % number of iteration
N_exp=100;                      % number of experimentation
step_size=1e-3;                 % step size 
mu=step_size*ones(N,1);         
sigma_u=1e0;                    % regression vector variation
Sigma_u=rand(N,1)+1;            % regression vector variation 
sigma_v=1e-3;                   % noise variation           
Sigma_v=sigma_v*ones(N,1);
r=0.075;                        % distance parameter for network topology
w_star=randn(L,1);              % local estimates initialisation
w_0=repmat(randn(L,1),1,N);     % desired parameter selection



%%


options.M=L;
options.N=N;
options.N_iter=N_iter;
options.parameter=r;
options.corr=0;
options.type='radial';
options.wo=w_star;
options.rule='metropolis';
options.sigma_u=sigma_u;
options.Sigma_u=Sigma_u;
options.sigma_v=sigma_v;
options.one_iter=0;
%%
[A,L1,Ac,xy] = get_network( options);
[ Coef ] = get_coef( A,Sigma_u,Sigma_v,mu,options );
C=Coef;
A=eye(N);




%% Monte-Carlo runs 


e_doubly_compressed=zeros(N_iter,N);

e_t=zeros(N_iter,N);


for l=1:N_exp
    [ d,u,Sigma_u,Sigma_v,w_star] = get_data( options );

    u=permute(u,[2 3 1]);
    
    H=zeros(L,N,N_iter);
    Q=zeros(L,N,N_iter);
    for i=1:N
        for j=1:N_iter
            H(:,i,j)=randperm(L);
            Q(:,i,j)=randperm(L);
        end
    end
    fprintf('exp n %d  \n',l);

    
    [e_t,w_k_doubly_compressed]=doubly_compressed_diffusion(A,C,w_0,u,d,mu,H,Q,M,M_grad,w_star);
    e_doubly_compressed=e_doubly_compressed+e_t;
end

MSD=mean(e_doubly_compressed,2)/N_exp;



%% theoretical model

fprintf('Theoritical model \n');

blK_sel_diag=kron(eye(N),ones(L));
C_i=kron(C,eye(L));
C_2blk=C_i'*C_i.*blK_sel_diag;
C_diag = diag(diag(C_i));
R_u=[];
R=[];
R_2=[];
R_c=[];
S=[];
R_v=[];
M_mu=[];

Ms=[];
R_t = zeros (N*L,N*L,N);
Rcm = zeros (N*L,N*L,N);



Ru=zeros(L,L,N);


for k=1:N
    Ru(:,:,k)=Sigma_u(k)*eye(L);
    R_u=blkdiag(R_u,Ru(:,:,k));
end

C3t=zeros(N);

for k = 1 : N
    for l = 1 : N
        for m= 1 : N
            C3t( k,l ) = C3t ( k,l ) + C( m , k ) * C( k , l ) * C( l , m  );
        end
    end
end

C3=kron(C3t,eye(L));

for i=1:N
    R_ut=0;
    R_2_t=0;
    Rt=[];
    Rct = [];
    for j=1:N
        R_ut=R_ut+C(j,i)*Ru(:,:,j);
        R_2_t=R_2_t +C(j,i)^2*Ru(:,:,j);
        Rt=blkdiag(Rt,Ru(:,:,j));
        Rct = blkdiag( Rct ,C( i,j ) * Ru(:,:,i));
    end
    M_mu=blkdiag(M_mu,mu(i)*eye(L));
    R=blkdiag(R,R_ut);
    R_2=blkdiag(R_2,R_2_t);
    R_c=blkdiag(R_c,sum(C(:,i))*Ru(:,:,i));
    R_v=blkdiag(R_v,Sigma_v(i));
    Ms=blkdiag(Ms,sum(C(:,i)));
    S=blkdiag(S,Sigma_v(i)*Ru(:,:,i));
    
    R_t(:,:,i) = Rt;
    Rcm(:,:,i) = Rct;
end


%% Mean square stability

alpha_1=(M/L)*((M-1)/(L-1)-(M/L));
alpha_2=(M/L)*(1-(M-1)/(L-1));
alpha_3=(M/L)^2;

beta_1=(M_grad/L)*((M_grad-1)/(L-1)-(M_grad/L));
beta_2=(M_grad/L)*(1-(M_grad-1)/(L-1));
beta_3=(M_grad/L)^2;

cg1=(M_grad/L)*(1-(M_grad-1)/(L-1));
cg2=(M_grad/L)*(M_grad-1)/(L-1);

c1=(M/L)*(1-(M-1)/(L-1));
c2=(M/L)*(M-1)/(L-1);

C2=C_i.*C_i;


H_p=eye(N*L);


%% y calculation

y1_1 = 0;
y1_2 = 0;
y1_3 = M_mu*C_i'*S*C_i*M_mu;

y2_1 = M_mu * S * C_i * M_mu;
y2_2 = M_mu * S * C3 * M_mu;
y2_3 = 0;


y4_1 = M_mu*S*C_2blk*M_mu;
y4_2 = 0;
y4_3 = M_mu*S*M_mu;

for m=1:N
    Tm=zeros(N);
    Tm(m,m)=1;
    Tm=kron(Tm,eye(L));
    
    y1_1 = y1_1 + M_mu * C_i' * Tm * S * Tm * C_i * M_mu;
    
end

for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    y1_2 = y1_2 + M_mu * C_i' * Dn * S * Dn * C_i * M_mu;
    y2_3 = y2_3 + M_mu * Dn * S * C3 * Dn * M_mu;
    y4_2 = y4_2 + M_mu * Dn * S* C_2blk * Dn * M_mu;
    
    
end

y1 = beta_1 *  y1_1 + beta_2 * y1_2 + beta_3 * y1_3;
y2 = M_grad / L * ( 1 - M_grad / L)* ( y2_1 - y2_2)...
    + M_grad / L * y2_2...
    - cg1 * y2_3 - cg2 * y2_2;
y3 = y2';
y4 = (1 - 2* M_grad / L ) * y4_1 ...
    + cg1 * y4_2 + cg2 * y4_1 ...
    + (1 -  M_grad / L )^2 * ( y4_3 - y4_1);

y = y1' + y2' + y3' + y4';
vec_y=y(:)';

%% p1 calculation

p1_1_1 = 0;
p1_1_2 = 0;
p1_1_3 = 0;
p1_1_4 = 0;
p1_2_1=0;

for n=1:N*L
    Dn=diag(H_p(:,n));
    p1_2_1 = p1_2_1 + kron(Dn * R * M_mu , Dn * R * M_mu);
end




for m=1:N
    Tm=zeros(N);
    Tm(m,m)=1;
    Tm=kron(Tm,eye(L));
    
    p1_1_4 = p1_1_4 + kron( Rcm(:,:,m) * M_mu , Rcm(:,:,m) * M_mu );
    
    
    for n=1:N*L
        Dn=diag(H_p(:,n));
        
        for o=1:N*L
            Do = diag(H_p(:,o));
            p1_1_1 = p1_1_1 + kron( Dn * Rcm(:,:,m) * Do * M_mu , Dn * Rcm(:,:,m) * Do * M_mu );
        end
        
        p1_1_2 = p1_1_2 + kron( Dn * Rcm(:,:,m) * M_mu , Dn * Rcm(:,:,m) * M_mu );
        p1_1_3 = p1_1_3 + kron( Rcm(:,:,m) * Dn *  M_mu , Rcm(:,:,m) * Dn * M_mu );
    end
    
    
end

p1_1 = c1 * ( cg1 * p1_1_1 + cg2 * p1_1_2 )...
    + c2 * ( cg1 * p1_1_3 + cg2 * p1_1_4 );


p1_2 = c1 * p1_2_1 + c2 * kron( R * M_mu , R * M_mu );

p1_3 = c1 * p1_1_2 + c2 * p1_1_4;

p1 = p1_1 + ( M_grad / L)^2 * ( p1_2 -  p1_3 );

%% p2 and p5 init

p2_1 = 0;
p2_2 = 0;
p2_3 = 0;
p2_4 = kron( R_u * M_mu , R * M_mu);
p2_5 = kron( R_u * M_mu , R_2 * M_mu);
p2_6 = 0;


p5_1 = 0;
p5_2 = 0;
p5_3 = 0;
p5_4 = kron( R * M_mu , R_u * M_mu );
p5_5 = kron( R_2 * M_mu , R_u * M_mu);
p5_6 = 0;

for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    p2_1  = p2_1 + kron( Dn * R_u * M_mu ,Dn * R * M_mu);
    p2_2  = p2_2 + kron( Dn * R_u * M_mu ,Dn * R_2 * M_mu);
    p2_6  = p2_6 + kron( R_u * Dn * M_mu ,R_2 * Dn * M_mu);
    
    p5_1  = p5_1 + kron( Dn * R * M_mu ,  Dn * R_u * M_mu );
    p5_2  = p5_2 + kron( Dn * R_2 * M_mu , Dn * R_u * M_mu);
    p5_6  = p5_6 + kron( R_2 * Dn * M_mu , R_u * Dn * M_mu );
    
    
    for m=1:N*L
        Dm=diag(H_p(:,m));
        p2_3  = p2_3 + kron( Dn * R_u * Dm * M_mu ,Dn * R_2 * Dm * M_mu);
        p5_3  = p5_3 + kron( Dn * R_2 * Dm * M_mu , Dn * R_u * Dm * M_mu );
    end
end

p2 = c1 * ( M_grad / L * ( 1 - M_grad / L ) * p2_1 ...
    + ( ( M_grad / L )^2 - cg2 ) * p2_2 ...
    - cg1 * p2_3)...
    + c2 * ( M_grad / L * ( 1 - M_grad / L ) * p2_4 ...
    + ( ( M_grad / L )^2 - cg2 ) * p2_5 ...
    - cg1 * p2_6);

p5 = c1 * ( M_grad / L * ( 1 - M_grad / L ) * p5_1 ...
    + ( ( M_grad / L )^2 - cg2 ) * p5_2 ...
    - cg1 * p5_3)...
    + c2 * ( M_grad / L * ( 1 - M_grad / L ) * p5_4 ...
    + ( ( M_grad / L )^2 - cg2 ) * p5_5 ...
    - cg1 * p5_6);


%% p3 and p9 calculation

p3_1_1=0;
p3_1_2=0;



p3_4_1=0;
p3_4_2=0;
p3_4_3=0;
p3_4_4=0;

p3_5_1=0;

p3_6_1=0;
p3_6_2=0;

p9_1_1=0;
p9_1_2=0;

p9_3=0;

p9_4_1=0;
p9_4_2=0;
p9_4_3=0;
p9_4_4=0;

p9_5_1=0;

p9_6_1=0;
p9_6_2=0;


for m=1:N
    Tm=zeros(N);
    Tm(m,m)=1;
    Tm=kron(Tm,eye(L));
    
    p3_1_2 = p3_1_2 + kron( R_u * Tm * C2 * M_mu , R_t(:,:,m) * M_mu);
    p9_1_2 = p9_1_2 + kron(R_t(:,:,m)*M_mu , R_u*Tm*C2*M_mu);
    p9_3   = p9_3   + kron(R_t(:,:,m)*M_mu , R_u*Tm*C2*M_mu);
    
    
    p3_4_4 = p3_4_4 + kron( R_u * Tm * C2  * M_mu , R_t(:,:,m) * M_mu);
    p9_4_4 = p9_4_4 + kron(R_t(:,:,m)*M_mu , R_u*Tm*C2*M_mu);
    
    p3_6_2 = p3_6_2 + kron(R_u*Tm*C2*M_mu,R_t(:,:,m)*M_mu);
    p9_6_2 = p9_6_2 + kron(R_t(:,:,m)*M_mu , R_u*Tm*C2*M_mu);
    
    for n=1:N*L
        Dn = diag(H_p(:,n));
        
        p3_1_1=p3_1_1 + kron(R_u * Tm * C2 * Dn * M_mu , R_t(:,:,m) * Dn * M_mu );
        p9_1_1=p9_1_1 + kron(R_t(:,:,m) * Dn * M_mu , R_u * Tm * C2 * Dn * M_mu);
        
        for o = n : L : N * L
            Do = diag(H_p(:,o));
            
            for p = 1 : N*L
                
                Dp = diag(H_p(:,p));
                
                p3_4_1= p3_4_1 + kron(Dn*R_u*Tm*C2*Dp*M_mu,Do*R_t(:,:,m)*Dp*M_mu)...
                    + kron(Do*R_u*Tm*C2*Dp*M_mu,Dn*R_t(:,:,m)*Dp*M_mu) ;
                
                p9_4_1= p9_4_1 + kron(Do*R_t(:,:,m)*Dp*M_mu , Dn*R_u*Tm*C2*Dp*M_mu)...
                    + kron(Dn*R_t(:,:,m)*Dp*M_mu , Do*R_u*Tm*C2*Dp*M_mu);
                
            end
            
            p3_4_3=p3_4_3 + kron(Dn*R_u*Tm*C2*M_mu,Do*R_t(:,:,m)*M_mu) ...
                + kron(Do*R_u*Tm*C2*M_mu,Dn*R_t(:,:,m)*M_mu);
            
            p3_6_1=p3_6_1 + kron(Dn*R_u*Tm*C2*M_mu,Do*R_t(:,:,m)*M_mu)...
                + kron(Do*R_u*Tm*C2*M_mu,Dn*R_t(:,:,m)*M_mu);
            
            p9_4_3=p9_4_3 + kron(Do*R_t(:,:,m)*M_mu , Dn*R_u*Tm*C2*M_mu) ...
                + kron(Dn*R_t(:,:,m)*M_mu , Do*R_u*Tm*C2*M_mu );
            
            p9_6_1=p9_6_1 + kron(Do*R_t(:,:,m)*M_mu , Dn*R_u*Tm*C2*M_mu)...
                + kron(Dn*R_t(:,:,m)*M_mu , Do*R_u*Tm*C2*M_mu );
            
        end
        
        
        
        for p = 1 : N*L
            Dp = diag(H_p(:,p));
            
            p3_4_1= p3_4_1 - kron(Dn*R_u*Tm*C2*Dp*M_mu,Dn*R_t(:,:,m)*Dp*M_mu);
            p9_4_1= p9_4_1 - kron(Dn*R_t(:,:,m)*Dp*M_mu , Dn*R_u*Tm*C2*Dp*M_mu);
        end
        
        p3_4_2 = p3_4_2 + kron(R_u*Tm*C2*Dn*M_mu,R_t(:,:,m)*Dn*M_mu);
        p9_4_2 = p9_4_2 + kron( R_t(:,:,m)*Dn*M_mu , R_u*Tm*C2*Dn*M_mu );
        
        p3_4_3= p3_4_3 - kron(Dn*R_u*Tm*C2*M_mu , Dn*R_t(:,:,m)*M_mu);
        p9_4_3= p9_4_3 - kron(Dn*R_t(:,:,m)*M_mu , Dn*R_u*Tm*C2*M_mu);
        
        p3_6_1= p3_6_1 - kron(Dn*R_u*Tm*C2*M_mu,Dn*R_t(:,:,m)*M_mu);
        p9_6_1= p9_6_1 - kron(Dn*R_t(:,:,m)*M_mu , Dn*R_u*Tm*C2*M_mu);
    end
    
    
end

for m=1:L*N
    Tm=diag(H_p(:,m));
    
    for n=m:L:L*N
        
        Tn=diag(H_p(:,n));
        
        p3_5_1 = p3_5_1 + kron( Tm * R_u* C_i*M_mu, Tn* R*M_mu ) ...
            + kron( Tn * R_u* C_i*M_mu, Tm* R*M_mu ) ;
        
        p9_5_1 = p9_5_1 + kron( Tn* R*M_mu , Tm * R_u* C_i*M_mu ) ...
            + kron( Tm * R*M_mu , Tn * R_u* C_i*M_mu ) ;
    end
    
    p3_5_1 = p3_5_1 - kron( Tm * R_u * C_i * M_mu, Tm * R * M_mu );
    p9_5_1 = p9_5_1 - kron( Tm* R*M_mu , Tm * R_u* C_i*M_mu );
    
end

p3_1 = ( M / L ) * ( cg1 * p3_1_1 + cg2 * p3_1_2 );
p3_2 = ( M / L ) * kron(R_u * C_i * M_mu, R  * M_mu);
p3_3 = p3_1_2;
p3_4=  cg1 * ( c1 * p3_4_1 + c2 * p3_4_2 ) + cg2 * ( c1 * p3_4_3 + c2 * p3_4_4 ) ;

p3_5_2 = kron( R_u  *C_i  *M_mu , R * M_mu );
p3_5 =  c1 * p3_5_1 + c2 * p3_5_2 ;

p3_6= c1 * p3_6_1 + c2 * p3_6_2 ;

p3= p3_1 - p3_4 +  (M_grad/L) ^2 *( p3_2 - p3_5 - ( M / L ) * p3_3  + p3_6 );

p9_1= ( M / L ) * ( cg1 * p9_1_1 +  cg2 * p9_1_2 );
p9_2= ( M / L ) * kron( R *M_mu , R_u * C_i *M_mu);
p9_4= cg1 * ( c1 * p9_4_1 + c2 * p9_4_2 ) + cg2 * ( c1 * p9_4_3 + c2 * p9_4_4 )  ;

p9_5_2 = kron( R * M_mu , R_u * C_i * M_mu );
p9_5 =  c1 * p9_5_1 + c2 * p9_5_2 ;

p9_6=  c1 * p9_6_1 + c2 * p9_6_2 ;

p9= p9_1 - p9_4 +  (M_grad/L) ^2 * ( p9_2 - p9_5 - ( M / L ) * p9_3   + p9_6 );
%% p4 and p13 calculation


p4_1= ( M * M_grad /(L^2) ) * kron( C_i* R_u * M_mu, R * M_mu );
p4_2_1= 0;
p4_2_2= 0;
p4_2_3= kron(C_i * R_u * M_mu , R * M_mu);

p4_3_1=0;

p4_4_1= 0;
p4_4_2= 0;
p4_4_3= 0;
p4_4_4= 0;
p4_4_5= 0;
p4_4_6= kron(C_i * R_u * M_mu,  R * M_mu );


p13_1= M*M_grad/(L^2) * kron( R * M_mu , C_i* R_u * M_mu);
p13_2_1= 0;
p13_2_2= 0;
p13_2_3= kron( R * M_mu , C_i * R_u * M_mu );

p13_3_1=0;

p13_4_1= 0;
p13_4_2= 0;
p13_4_3= 0;
p13_4_4= 0;
p13_4_5= 0;
p13_4_6= kron( R * M_mu , C_i * R_u * M_mu);



for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    p4_3_1  = p4_3_1 + kron(C_i * Dn * R_u * M_mu, Dn * R * M_mu );
    p13_3_1 = p13_3_1 + kron( Dn * R * M_mu , C_i * Dn * R_u * M_mu );
    
end




for m=1:N
    Tm=zeros(N);
    Tm(m,m)=1;
    Tm=kron(Tm,eye(L));
    
    for n=1:N*L
        Dn = diag(H_p(:,n));
        
        for o = n : L : N * L
            Do = diag(H_p(:,o));
            
            for p = 1 : N*L
                Dp = diag(H_p(:,p));
                
                p4_4_1= p4_4_1 + kron(Dn*Tm*C2*R_u*Dp*M_mu,Do*R_t(:,:,m)*Dp*M_mu)...
                    + kron(Do*Tm*C2*R_u*Dp*M_mu,Dn*R_t(:,:,m)*Dp*M_mu);
                
                p13_4_1= p13_4_1 + kron(Do*R_t(:,:,m)*Dp*M_mu , Dn*Tm*C2*R_u*Dp*M_mu)...
                    + kron(Dn*R_t(:,:,m)*Dp*M_mu , Do*Tm*C2*R_u*Dp*M_mu);
                
            end
            
            p4_4_2= p4_4_2 + kron(Dn*Tm*C2*R_u*M_mu,Do*R_t(:,:,m)*M_mu)...
                + kron(Do*Tm*C2*R_u*M_mu,Dn*R_t(:,:,m)*M_mu);
            
            p13_4_2= p13_4_2 + kron(Do*R_t(:,:,m)*M_mu , Dn*Tm*C2*R_u*M_mu)...
                + kron(Dn*R_t(:,:,m)*M_mu , Do*Tm*C2*R_u*M_mu);
            
        end
        
        for p = 1 : N*L
            Dp = diag(H_p(:,p));
            
            p4_4_1= p4_4_1 - kron(Dn*Tm*R_u*C2*Dp*M_mu,Dn*R_t(:,:,m)*Dp*M_mu);
            p13_4_1= p13_4_1 - kron(Dn*R_t(:,:,m)*Dp*M_mu , Dn*Tm*R_u*C2*Dp*M_mu);
        end
        
        p4_2_1= p4_2_1 + kron(Tm * C2 * R_u * Dn * M_mu , R_t(:,:,m) * Dn * M_mu);
        p13_2_1= p13_2_1 + kron( R_t(:,:,m) * Dn * M_mu , Tm * C2 * R_u * Dn * M_mu );
        
        
        p4_4_3= p4_4_3 + kron(Tm*C2*R_u*Dn*M_mu,R_t(:,:,m)*Dn*M_mu);
        p13_4_3= p13_4_3 + kron(R_t(:,:,m)*Dn*M_mu , Tm*C2*R_u*Dn*M_mu);
        
        p4_4_2= p4_4_2 - kron(Dn*Tm*C2*R_u*M_mu , Dn*R_t(:,:,m)*M_mu);
        p13_4_2= p13_4_2 - kron(Dn*R_t(:,:,m)*M_mu , Dn*Tm*C2*R_u*M_mu);
        
        
        
        
    end
    
    
    p4_2_2= p4_2_2 + kron(Tm * C2 * R_u  * M_mu , R_t(:,:,m) *  M_mu);
    p13_2_2= p13_2_2 + kron( R_t(:,:,m) *  M_mu , Tm * C2 * R_u  * M_mu );
    
    
    
    p4_4_4 = p4_4_4 + kron(Tm*C2*R_u*M_mu,R_t(:,:,m)*M_mu);
    p13_4_4 = p13_4_4 + kron( R_t(:,:,m)*M_mu , Tm*C2*R_u*M_mu );
    
    
end



for m=1:L*N
    Tm=diag(H_p(:,m));
    for n=m:L:L*N
        Tn=diag(H_p(:,n));
        
        p4_4_5 = p4_4_5 + kron( Tm * C_i * R_u * M_mu, Tn * R * M_mu )...
            + kron( Tn * C_i * R_u * M_mu, Tm * R * M_mu );
        
        p13_4_5 = p13_4_5 + kron( Tn * R * M_mu , Tm * C_i * R_u * M_mu )...
            + kron( Tm * R * M_mu , Tn * C_i * R_u * M_mu );
        
    end
    
    
    p4_4_5 = p4_4_5 - kron( Tm * C_i * R_u * M_mu, Tm * R * M_mu );
    p13_4_5 = p13_4_5 - kron( Tm * R * M_mu , Tm * C_i * R_u * M_mu );
    
end

%p4_4_5=p4_3_1;
%p13_4_5=p13_3_1;

p4_2 = ( M/L ) * ( cg1 * p4_2_1 + cg2 * p4_2_2 + ( M_grad/L )^2 * ( p4_2_3 - p4_2_2 ) );
p13_2 = ( M/L ) * ( cg1 * p13_2_1 + cg2 * p13_2_2 + ( M_grad/L )^2 * ( p13_2_3 - p13_2_2 ) );

p4_3_2= kron(C_i * R_u * M_mu, R * M_mu);
p13_3_2= kron( R * M_mu , C_i * R_u * M_mu);

p4_3=  (M_grad/L)*( c1 * p4_3_1 + c2 * p4_3_2  );
p13_3=  (M_grad/L)*( c1*p13_3_1 + c2* p13_3_2  );


p4_4= c1 * ( cg1 * p4_4_1  + cg2 * p4_4_2 + ( M_grad/ L )^2 * (p4_4_5 - p4_4_2) )...
    + c2 * ( cg1 * p4_4_3  + cg2 * p4_4_4 + ( M_grad/ L )^2 * (p4_4_6 - p4_4_4) );

p13_4= c1 * ( cg1 * p13_4_1  + cg2 * p13_4_2 + ( M_grad/ L )^2 * (p13_4_5 - p13_4_2)  )...
    + c2  * ( cg1 * p13_4_3  + cg2 * p13_4_4 + ( M_grad/ L )^2 * (p13_4_6 - p13_4_4)  );

p4= p4_1 - p4_2 - p4_3 + p4_4;
p13= p13_1 - p13_2 - p13_3 + p13_4;

%% p6 calculation

p6_1=0;
p6_2=kron( R_u *  M_mu, R_u * C_2blk *  M_mu );
p6_3=kron( R_u *  M_mu, R_u *  M_mu );
p6_4=0;
p6_5=0;
p6_6=0;

for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    p6_1 = p6_1 + kron( R_u * Dn *  M_mu , R_u * C_2blk * Dn * M_mu  );
    
    
    for m=1:N*L
        Dm=diag(H_p(:,m));
        
        p6_4 = p6_4 + kron( Dn * R_u * Dm * M_mu,  Dn  *  R_u * C_2blk *Dm * M_mu  );
    end
    
    p6_5 = p6_5 + kron( Dn * R_u *  M_mu , Dn * R_u  * C_2blk * M_mu );
    p6_6 = p6_6 + kron( Dn * R_u *  M_mu , Dn * R_u  * M_mu );
end


p6= (alpha_1 + alpha_3)*( cg1 * p6_1...
    + ( cg2 -(M_grad /L)^2 ) * p6_2 ...
    + ( 1 - M_grad/L )^2 * p6_3)...
    +alpha_2 *( cg1 * p6_4...
    + ( cg2 -(M_grad /L)^2 ) * p6_5 ...
    + ( 1 - M_grad/L )^2 * p6_6);

%% p7 and P10 calculation


p7_1 = M * M_grad/ (L^2) * kron( R_u* C_i * M_mu , R_u * M_mu  );
p7_2_1=0;
p7_3_1=0;
p7_4_1_1=0;
p7_4_1_2=0;
p7_4_1_3=0;
p7_4_2_1=0;

p10_1 = M * M_grad/ (L^2) * kron( R_u * M_mu , R_u* C_i * M_mu  );
p10_2_1=0;
p10_3_1=0;
p10_4_1_1=0;
p10_4_1_2=0;
p10_4_1_3=0;
p10_4_2_1=0;


for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    p7_2_1 = p7_2_1 + kron( R_u * C2 * Dn * M_mu , R_u * Dn * M_mu );
    p10_2_1 = p10_2_1 + kron(  R_u * Dn * M_mu , R_u * C2 * Dn * M_mu );
    
    p7_4_1_3= p7_4_1_3 + kron( R_u * C2 * Dn * M_mu , R_u * Dn * M_mu );
    p10_4_1_3= p10_4_1_3 + kron( R_u * Dn * M_mu , R_u * C2 * Dn * M_mu );
    
end





for m=1:L*N
    Tm=diag(H_p(:,m));
    
    for n=m:L:L*N
        Tn=diag(H_p(:,n));
        
        p7_3_1 = p7_3_1 + kron( Tm * R_u * C_i * M_mu , Tn * R_u *M_mu )...
            + kron( Tn * R_u * C_i * M_mu , Tm * R_u * M_mu );
        
        p10_3_1 = p10_3_1 + kron( Tn * R_u *M_mu , Tm * R_u * C_i * M_mu )...
            + kron( Tm * R_u *M_mu , Tn * R_u * C_i * M_mu );
        
        p7_4_1_2 = p7_4_1_2 + kron( Tm * R_u * C2  * M_mu , Tn * R_u  * M_mu )...
            + kron( Tn * R_u * C2  * M_mu , Tm * R_u  * M_mu );
        
        p10_4_1_2 = p10_4_1_2 + kron(  Tn * R_u  * M_mu , Tm * R_u * C2  * M_mu )...
            + kron( Tm * R_u  * M_mu , Tn * R_u * C2  * M_mu );
        
        p7_4_2_1 = p7_4_2_1 + kron( Tm * R_u * C_i * M_mu , Tn * R_u  * M_mu )...
            + kron( Tn * R_u * C_i * M_mu , Tm * R_u  * M_mu );
        
        p10_4_2_1 = p10_4_2_1 + kron( Tn * R_u  * M_mu , Tm * R_u * C_i * M_mu )...
            + kron( Tm * R_u  * M_mu , Tn * R_u * C_i * M_mu );
        
        
        for o = 1 : N* L
            Do=diag(H_p(:,o));
            
            p7_4_1_1= p7_4_1_1 + kron( Tm * R_u * C2 * Do * M_mu , Tn * R_u * Do * M_mu )...
                + kron( Tn * R_u * C2 * Do * M_mu , Tm * R_u * Do * M_mu );
            
            p10_4_1_1= p10_4_1_1 + kron( Tn * R_u * Do * M_mu , Tm * R_u * C2 * Do * M_mu )...
                + kron( Tm * R_u * Do * M_mu , Tn * R_u * C2 * Do * M_mu  );
            
        end
        
        
    end
    
    p7_3_1 = p7_3_1 - kron( Tm * R_u * C_i * M_mu , Tm * R_u * M_mu );
    p10_3_1 = p10_3_1 - kron( Tm * R_u *M_mu , Tm * R_u * C_i * M_mu );
    
    p7_4_1_2 = p7_4_1_2 - kron( Tm * R_u * C2  * M_mu , Tm * R_u  * M_mu );
    p10_4_1_2 = p10_4_1_2 - kron(  Tm * R_u  * M_mu , Tm * R_u * C2  * M_mu );
    
    p7_4_2_1 = p7_4_2_1 - kron( Tm * R_u * C_i * M_mu , Tm * R_u  * M_mu );
    p10_4_2_1 = p10_4_2_1 - kron( Tm * R_u  * M_mu , Tm * R_u * C_i * M_mu );
    
    
    
    
    
    
    
    
    for o = 1 : N* L
        Do=diag(H_p(:,o));
        
        p7_4_1_1 = p7_4_1_1 - kron( Tm * R_u * C2 * Do * M_mu , Tm * R_u * Do * M_mu );
        p10_4_1_1= p10_4_1_1 - kron( Tm * R_u * Do * M_mu , Tm * R_u * C2 * Do * M_mu );
        
    end
    
    
end

p7_2 = ( M/L ) * ( cg1 * p7_2_1 + cg2 * kron( R_u * C2 *  M_mu , R_u  * M_mu ) ...
    +( M_grad/L )^2 *( kron( R_u * C_i *  M_mu , R_u *  M_mu ) - kron( R_u * C2  * M_mu , R_u *  M_mu ) ) );

p10_2 = ( M/L ) * ( cg1 * p10_2_1 + cg2 * kron( R_u  * M_mu , R_u * C2 *  M_mu  ) ...
    +( M_grad/L )^2 *( kron(  R_u *  M_mu , R_u * C_i *  M_mu ) - kron( R_u  * M_mu , R_u * C2 *  M_mu  ) ) );

p7_3 =   ( M_grad/L ) * ( c1 * p7_3_1 + c2 * kron( R_u * C_i  * M_mu , R_u *  M_mu ));
p10_3 =  ( M_grad/L ) * ( c1 * p10_3_1 + c2 * kron( R_u *  M_mu , R_u * C_i  * M_mu  ));

p7_4_1= c1 * ( cg1 * p7_4_1_1 + cg2 * p7_4_1_2  ) ...
    + c2 * ( cg1 * p7_4_1_3 + cg2 * kron( R_u * C2 *  M_mu , R_u * M_mu )  );

p10_4_1= c1 * ( cg1 * p10_4_1_1 + cg2 * p10_4_1_2  ) ...
    + c2 * ( cg1 * p10_4_1_3 + cg2 * kron( R_u * M_mu ,R_u * C2 *  M_mu )  );

p7_4_2_2 = kron( R_u * C_i  * M_mu ,  R_u  * M_mu );
p10_4_2_2 = kron( R_u  * M_mu , R_u * C_i  * M_mu );

p7_4_2= c1 * p7_4_2_1 + c2 * p7_4_2_2;
p10_4_2= c1 * p10_4_2_1 + c2 * p10_4_2_2;

p7_4_3= c1 * p7_4_1_2 + c2 * kron( R_u * C2  * M_mu ,  R_u  * M_mu );
p10_4_3= c1 * p10_4_1_2 + c2 * kron( R_u  * M_mu , R_u * C2  * M_mu );

p7_4 = p7_4_1 + ( M_grad / L )^2 *( p7_4_2 - p7_4_3 );
p10_4 = p10_4_1 + ( M_grad / L )^2 *( p10_4_2 - p10_4_3 );

p7= p7_1 - p7_2 - p7_3 + p7_4;
p10= p10_1 - p10_2 - p10_3 + p10_4;

%% p8 and p14 calculation


p8_1 =  kron(C_i * R_u *M_mu ,R_u * M_mu);
p8_2 = 0;
p8_3 = 0;
p8_4 = kron( C2 * R_u *  M_mu , R_u *  M_mu );
p8_5 = 0;
p8_6 = 0;


p14_1 = kron(R_u * M_mu, C_i * R_u *M_mu);
p14_2 = 0;
p14_3 = 0;
p14_4 = kron( R_u *  M_mu , C2 * R_u *  M_mu );
p14_5 = 0;
p14_6 = 0;



for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    p8_2  = p8_2 + kron( C_i  * Dn * R_u  *  M_mu , Dn * R_u *  M_mu );
    p14_2 = p14_2 + kron( Dn * R_u *  M_mu , C_i  * Dn * R_u  *  M_mu );
    
    p8_3  = p8_3 + kron( C2 * R_u * Dn *  M_mu , R_u * Dn *  M_mu );
    p14_3 = p14_3 + kron( R_u * Dn *  M_mu , C2 * R_u * Dn *  M_mu );
    
    for m= 1 : N*L
        Dm=diag(H_p(:,m));
        
        p8_5  = p8_5 + kron( C2 * Dm * R_u * Dn *  M_mu , Dm *  R_u * Dn *  M_mu );
        p14_5 = p14_5 + kron( Dm *  R_u * Dn *  M_mu , C2 * Dm *  R_u * Dn *  M_mu );
    end
    
    p8_6  = p8_6 + kron( C2 * Dn * R_u *  M_mu , Dn *  R_u  *  M_mu );
    p14_6 = p14_6 + kron( Dn *  R_u *  M_mu , C2 * Dn *  R_u *  M_mu );
    
end




p_8_1  =  M/L * p8_1;
p_14_1 = M/L * p14_1;


p_8_2  = c1 * p8_2 + c2 * p8_1;
p_14_2 = c1 * p14_2 + c2 * p14_1;

p_8_3  = cg1 * p8_3 + cg2 * p8_4...
    + ( M_grad / L )^2 * ( p8_1 - p8_4 );

p_14_3 = cg1 * p14_3 + cg2 * p14_4...
    + ( M_grad / L )^2 * ( p14_1 - p14_4 );

p_8_4  = c1 * ( cg1 * p8_5 + cg2 * p8_6 + ( M_grad / L )^2 * ( p8_2 - p8_6 ))...
    + c2 * ( cg1 * p8_3 + cg2 * p8_4 +( M_grad / L )^2 * ( p8_1 - p8_4 ));

p_14_4  = c1 * ( cg1 * p14_5 + cg2 * p14_6 + ( M_grad / L )^2 * ( p14_2 - p14_6 ))...
    + c2 * ( cg1 * p14_3 + cg2 * p14_4 +( M_grad / L )^2 * ( p14_1 - p14_4 ));

p8 = (1 - 2* M_grad/L) *( p_8_1 - p_8_2 ) + (M/L) * p_8_3 - p_8_4;
p14 = (1 - 2* M_grad/L) *( p_14_1 - p_14_2 ) + (M/L) * p_14_3 - p_14_4;
%% p11 calculation

p11_1=0;
p11_2=kron(  R_u * C_i * M_mu ,R_u * C_i *  M_mu );
p11_3=0;
p11_4=0;
p11_5=0;
p11_6=0;


for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    p11_1 = p11_1 +  kron(  R_u * Dn * C_i * M_mu , R_u * Dn  * C_i' *  M_mu );
    
end

for m=1:N
    
    Tm=zeros(N);
    Tm(m,m)=1;
    Tm=kron(Tm,eye(L));
    
    p11_5 = p11_5 +  kron(  R_u * Tm * C_i * M_mu , R_u * Tm  * C_i' *  M_mu );
    
end



for m=1:L*N
    Tm=diag(H_p(:,m));
    
    for n=m:L:L*N
        Tn=diag(H_p(:,n));
        
        for o=1: N*L
            Do=diag(H_p(:,o));
            
            p11_3 = p11_3 +  kron(  Tm * R_u * Do * C_i * M_mu ,Tn * R_u * Do * C_i' *  M_mu )...
                + kron(  Tn * R_u * Do * C_i * M_mu ,Tm * R_u * Do * C_i' *  M_mu );
        end
        
        p11_4 = p11_4 +  kron(  Tm * R_u * C_i * M_mu ,Tn * R_u * C_i' *  M_mu )...
            + kron(  Tn * R_u *  C_i * M_mu ,Tm * R_u *  C_i' *  M_mu );
        
        for o=1:N
            
            To=zeros(N);
            To(o,o)=1;
            To=kron(To,eye(L));
            
            p11_6 = p11_6 +  kron(  Tm * R_u * To * C_i * M_mu ,Tn * R_u * To * C_i' *  M_mu )...
                + kron(  Tn * R_u * To * C_i * M_mu ,Tm * R_u * To * C_i' *  M_mu );
        end
        
    end
    
    for o=1: N*L
        Do=diag(H_p(:,o));
        
        p11_3 = p11_3 -  kron(  Tm * R_u * Do * C_i * M_mu ,Tm * R_u * Do * C_i' *  M_mu );
    end
    p11_4 = p11_4 -  kron(  Tm * R_u * C_i * M_mu ,Tm * R_u * C_i' *  M_mu );
    
    for o=1:N
        
        To=zeros(N);
        To(o,o)=1;
        To=kron(To,eye(L));
        
        p11_6 = p11_6  - kron(  Tm * R_u * To * C_i * M_mu ,Tm * R_u * To * C_i' *  M_mu );
            
    end
    
end



p11 = ( 1 - 2 * M/L ) * ( beta_1 * p11_5 + beta_2 * p11_1 + beta_3 * p11_2 ) ...
    + c1 * ( beta_1 * p11_5 + beta_2 * p11_3 + beta_3 * p11_4 )...
    + c2 * ( beta_1 * p11_6 + beta_2 * p11_1 + beta_3 * p11_2 );


%% p12 and p15 calculation


p12_1=kron( C_i * R_u * M_mu , R_u * C_i * M_mu );
p12_2=0;
p12_3=0;
p12_4=0;
p12_5=0;
p12_6=0;

p15_1=kron( R_u * C_i * M_mu , C_i * R_u * M_mu );
p15_2=0;
p15_3=0;
p15_4=0;
p15_5=0;
p15_6=0;



for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    for m=1:N*L
        Dm=diag(H_p(:,m));
        
        p12_5 = p12_5 + kron(Dn * C_i * R_u * Dm *  M_mu ,Dn * R_u * C_i * Dm * M_mu );
        p15_5 = p15_5 + kron(Dn * R_u * C_i * Dm * M_mu , Dn * C_i * R_u * Dm *  M_mu );
        
    end
    
    
    
    p12_6 = p12_6 + kron(Dn * C_i * R_u *  M_mu ,Dn * R_u * C_i * M_mu );
    p15_6 = p15_6 + kron(Dn * R_u * C_i * M_mu ,Dn * C_i * R_u *  M_mu );
    
    
end




for m=1:N
    Tm=zeros(N);
    Tm(m,m)=1;
    Tm=kron(Tm,eye(L));
    
    
    for n=1:N*L
        Dn=diag(H_p(:,n));
        
        p12_2 = p12_2 + kron(Tm * C_i' * R_u * Dn *  M_mu ,Tm * R_u * C_i * Dn * M_mu );
        p15_2 = p15_2 + kron(Tm * R_u * C_i * Dn * M_mu ,Tm * C_i' * R_u * Dn *  M_mu  );
        
        
    end
    
    p12_3 = p12_3 + kron(Tm * C_i' * R_u * M_mu ,Tm * R_u * C_i * M_mu );
    p15_3 = p15_3 + kron(Tm * R_u * C_i * M_mu ,Tm * C_i' * R_u * M_mu );
    
    
end

for m=1:L*N
    Tm=diag(H_p(:,m));
    
    for n=m:L:L*N
        Tn=diag(H_p(:,n));
        
        p12_4 = p12_4 + kron(Tm * C_i * R_u * M_mu ,Tn * R_u * C_i * M_mu )...
            + kron(Tn * C_i * R_u * M_mu ,Tm * R_u * C_i * M_mu );
        
        p15_4 = p15_4 + kron(Tn * R_u * C_i * M_mu ,Tm * C_i * R_u * M_mu )...
            + kron(Tm * R_u * C_i * M_mu , Tn * C_i * R_u * M_mu );
        
        
    end
    
    p12_4 = p12_4 - kron(Tm * C_i * R_u * M_mu ,Tm * R_u * C_i * M_mu );
    p15_4 = p15_4 - kron(Tm * R_u * C_i * M_mu ,Tm * C_i * R_u * M_mu );
    
end

p_12_1 = ( M_grad / L ) * p12_1;
p_15_1 = ( M_grad / L ) * p15_1;

p_12_2 = cg1 * p12_2 + ( cg2 - (M_grad/L)^2) * p12_3 +  (M_grad/L)^2 * p12_1;
p_15_2 = cg1 * p15_2 + ( cg2 - (M_grad/L)^2) * p15_3 +  (M_grad/L)^2 * p15_1;

p_12_3 = ( M_grad / L) * ( c1 * p12_4 + c2 * p12_1 );
p_15_3 = ( M_grad / L) * ( c1 * p15_4 + c2 * p15_1 );

p_12_4= c1 * (cg1 * p12_5 + ( cg2 - (M_grad/L)^2) * p12_6 +  (M_grad/L)^2 * p12_4 )...
    + c2 * (cg1 * p12_2 + ( cg2 - (M_grad/L)^2) * p12_3 +  (M_grad/L)^2 * p12_1 );

p_15_4= c1 * (cg1 * p15_5 + ( cg2 - (M_grad/L)^2) * p15_6 +  (M_grad/L)^2 * p15_4 )...
    + c2 * (cg1 * p15_2 + ( cg2 - (M_grad/L)^2) * p15_3 +  (M_grad/L)^2 * p15_1 );

p12=( 1 - 2 * M /L ) * (p_12_1 - p_12_2)...
    + p_12_3 - p_12_4;

p15=( 1 - 2 * M /L ) * (p_15_1 - p_15_2)...
    + p_15_3 - p_15_4;

%% p16 calculation
p16_1=kron( C_i * R_u * M_mu , C_i * R_u * M_mu );
p16_2=0;
p16_3=0;
p16_4=0;
p16_5=0;
p16_6=0;

for n=1:N*L
    
    Dn=diag(H_p(:,n));
    
    p16_4 = p16_4 +  kron( C_i * Dn * R_u * M_mu , C_i * Dn * R_u * M_mu );
    
    
end


for m=1:N
    Tm=zeros(N);
    Tm(m,m)=1;
    Tm=kron(Tm,eye(L));
    
    for n=1:N*L
        Dn=diag(H_p(:,n));
        p16_2 = p16_2 +  kron( Tm * C_i * R_u * Dn * M_mu ,Tm * C_i * R_u * Dn * M_mu );
        
        for o = 1:N*L
            Do=diag(H_p(:,o));
            p16_5 = p16_5 +  kron( Tm * C_i * Do * R_u * Dn * M_mu ,Tm * C_i * Do * R_u * Dn * M_mu );
        end
        
        p16_6 = p16_6 +  kron( Tm * C_i * Dn * R_u * M_mu ,Tm * C_i * Dn * R_u * M_mu );
    end
    
    p16_3 = p16_3 +  kron( Tm * C_i * R_u * M_mu ,Tm * C_i * R_u * M_mu );
end

p_16_1 =(1 - 2 * M_grad / L) * p16_1;
p_16_2 = cg1 * p16_2 + ( cg2 - ( M_grad / L )^2 ) * p16_3 + ( M_grad / L )^2 * p16_1;
p_16_3 = (1 - 2 * M_grad / L ) * ( c1 * p16_4 + c2 * p16_1 );
p_16_4 = cg1 * ( c1 * p16_5 + c2 * p16_2 )...
    + ( cg2 - ( M_grad / L)^2 ) * ( c1 * p16_6 +  c2 * p16_3 )...
    + ( M_grad / L )^2 * ( c1 * p16_4 + c2 * p16_1);  %% M au lieu de M_grad suit mieux pour M=1

p16 = ( 1 - 2 * M / L) *(  p_16_1 + p_16_2 )...
    +  p_16_3 + p_16_4;

%% Theoretical model calculation



I_NM=eye(L*N);



F = eye((N*L)^2) - M * M_grad / L^2 * kron( R * M_mu , I_NM ) - M / L *( 1 - M_grad/ L) * kron( R_u * M_mu , I_NM )...
    - M_grad / L * ( 1 - M / L ) * kron( R_u * C_i * M_mu , I_NM )...
    - ( 1 - M / L ) * ( 1 - M_grad / L ) * kron( C_i * R_u * M_mu , I_NM )...
    - M * M_grad / L^2 * kron( I_NM , R * M_mu ) - M / L *( 1 - M_grad/ L) * kron( I_NM , R_u * M_mu )...
    - M_grad / L * ( 1 - M / L ) * kron( I_NM , R_u * C_i' * M_mu )...
    - ( 1 - M / L ) * ( 1 - M_grad / L ) * kron( I_NM , C_i * R_u * M_mu )...
    + p1 + p2 + p3 + p4 + p5 + p6 + p7 + p8 + p9 + p10 + p11 + p12 + p13 + p14 + p15 + p16;

F=sparse(F);

sig=eye(N*L);
epsilon=zeros(N_iter,1);

wt_1=repmat(w_star,1,N) - w_0;
epsilon_t=(1/N)*wt_1(:)'*reshape(F*sig(:),N*L,N*L)*wt_1(:)+(1/N)*vec_y*sig(:);
epsilon(1)=epsilon_t;

Fi=F;


for i=2:N_iter
    
    fprintf('iter n %d  \n',i);
    
    Met=(eye((N*L)^2)-F)*Fi*sig(:);
    Met=reshape(Met,N*L,N*L);
    
    
    
    epsilon_t=epsilon_t+(1/N)*vec_y*Fi*sig(:)-(1/N)*wt_1(:)'*Met*wt_1(:);
    epsilon(i)=epsilon_t;
    
    Fi=Fi*F;
end

MSD_t=(1/N)*vec_y*(eye(size(F))-F)^(-1)*reshape(eye(N*L),[],1);
EMSE=(1/N)*vec_y*(eye(size(F))-F)^(-1)*R(:);



%% MSD_curves

clf
fs=16;
plot(10*log10([  epsilon MSD_t*ones(N_iter,1) MSD])); grid on
axis([0 N_iter -70 20])
legend( 'Theoretical MSD', 'Simulated MSD', 'Steady state MSD')
title({'Doubly-compressed diffusion LMS'},...
    'Interpreter','latex','FontSize',1.2*fs)
set(gca,'fontsize',fs)
xlabel('iteration','fontsize',20,'interpreter','Latex');
ylabel('MSD (dB)','fontsize',20,'interpreter','Latex');






