function [A,L,Ac,xy] = get_network( options)
%
%
% Please cite the paper referenced below if you use this code.
% @article{harranedoubly,
%   title={Doubly compressed diffusion LMS over adaptive networks},
%   author={Harrane, Ibrahim El Khalil and Flamary, R{\'e}mi and Richard, C{\'e}dric}
%
%%
%
%This function generates a network and returns the following parameters:

%         A:            The matrix that defines the nodes connection
%         L:            Laplacian matrix
%         An:           Algebraic connectivity
%         xy:           Nodes coordinate
%
% Inputs:
%         options.N:            Number of nodes                     (default=10)
%         options.type:         Topology type: binomial or radial   (default='radial')
%         options.parameter:    Probablility for binomila topology and radius for (default=0.2)
%                               radial topology



%% Network generation
options=initoptions('get_network',options);

N=options.N;
type=options.type;
parameter=options.parameter;

A=zeros(N);       % topology matrix
L=zeros(N);       % Laplacian matrix






Ac=0;
while(Ac<1e-4)
    
    % Generating random coordinate
    
  
        x=randn(N,1) + 0.1;
        y=randn(N,1) + 0.1;
        xy=[x y];
  
    
    
    switch type
        case 'binomial'
            p=parameter;
            % Linking the nodes randomly
            
            for i=1:length(x)
                A(i,i)=1;
                for j=i+1:length(y)
                    pn=rand(1,1);
                    if (pn>p)
                        A(i,j)=1;
                        A(j,i)=1;
                    end
                end
            end
            
            
        case 'radial'
            r=parameter;
            for i=1:length(x)
                
                for j=1:length(y)
                    rn=norm([x(j)-x(i) y(j)-y(i)],2);
                    A(i,j)=(rn<=r);
                end
            end
            
        case 'gaussian'
            
            for i=1:length(x)
                A(i,i)=1;
                for j=i+1:length(y)
                    sigma=1;
                    mu=0.1;
                    p=parameter;
                    rn=norm([x(j)-x(i) y(j)-y(i)],2);
                    pn=(1/(sigma*sqrt(2*pi)))*exp(-(1/2)*(rn-mu)^2/sigma^2);
                    
                    if (pn>p)
                        A(i,j)=1;
                        A(j,i)=1;
                    end
                end
            end
            
    end
    
    %% Algebric Connectivity
    
    for i=1:N
        L(i,i)=max(0,(sum(A(:,i))-1));
        for j=i+1:N
            L(i,j)=-1*A(i,j);
            L(j,i)=-1*A(j,i);
        end
    end
    
    lambda=eig(L);
    Ac=lambda(2);
    
end


end

